@@ -1,4 +1,4 @@
-// $Id: SpTraceVcdC.cpp 49154 2008-01-02 14:22:02Z wsnyder $ -*- SystemC -*-
+// $Id: SpTraceVcdC.cpp 61048 2008-09-18 00:45:49Z wsnyder $ -*- SystemC -*-
 //=============================================================================
 //
 // THIS MODULE IS PUBLICLY LICENSED
@@ -21,20 +21,21 @@
 ///
 //=============================================================================
 
-#include <time.h>
+#include <ctime>
 #include <iostream>
 #include <fstream>
-#include <assert.h>
+#include <cassert>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <string.h>
 #if defined(_WIN32) && !defined(__MINGW32__) && !defined(__CYGWIN__)
 # include <io.h>
 #else
 # include <unistd.h>
 #endif
-#include <errno.h>
-#include <stdio.h>
+#include <cerrno>
+#include <cstdio>
 
 // Note cannot include systemperl.h, or we won't work with non-SystemC compiles
 #include "SpCommon.h"
@@ -166,6 +167,16 @@ void SpTraceVcd::closePrev () {
     ::close(m_fd);
 }
 
+void SpTraceVcd::closeErr () {
+    // Close due to an error.  We might abort before even getting here,
+    // depending on the definition of SP_ABORT.
+    if (!isOpen()) return;
+
+    // No buffer flush, just fclose
+    m_isOpen = false;
+    ::close(m_fd);  // May get error, just ignore it
+}
+
 void SpTraceVcd::close() {
     if (!isOpen()) return;
     if (m_evcd) {
@@ -221,8 +232,9 @@ void SpTraceVcd::bufferFlush () {
 	    m_wroteBytes += got;
 	} else if (got < 0) {
 	    if (errno != EAGAIN && errno != EINTR) {
-		/* write failed, presume error */
-		perror("bin-write");
+		/* write failed, presume error (perhaps out of disk space) */
+		SP_ABORT("%Error: SpTraceVcd::bufferFlush: "<<strerror(errno)<<endl);
+		closeErr();
 		break;
 	    }
 	}
@@ -266,7 +278,7 @@ double SpTraceVcd::timescaleToDouble (const char* unitp) {
 }
 
 string SpTraceVcd::doubleToTimescale (double value) {
-    char* suffixp = "s";
+    const char* suffixp = "s";
     if	    (value>=1e0)   { suffixp="s"; value *= 1e0; }
     else if (value>=1e-3 ) { suffixp="ms"; value *= 1e3; }
     else if (value>=1e-6 ) { suffixp="us"; value *= 1e6; }
@@ -378,7 +390,7 @@ void SpTraceVcd::module (string name) {
 }
 
 void SpTraceVcd::declare (uint32_t code, const char* name, int arraynum,
-			  int msb, int lsb) {	// -1 = is boolean
+			  bool bussed, int msb, int lsb) {
     if (!code) { SP_ABORT("%Error: internal trace problem, code 0 is illegal\n"); }
 
     // Make sure array is large enough
@@ -416,29 +428,29 @@ void SpTraceVcd::declare (uint32_t code, const char* name, int arraynum,
     } else {
 	decl += stringCode(code);
     }
-    decl += (string(" "))+basename;
+    decl += " ";
+    decl += basename;
     if (arraynum>=0) {
 	sprintf(buf, "(%d)", arraynum);
 	decl += buf;
 	hiername += buf;
     }
-    if (msb<0) {
-	decl += " $end\n";
-    } else {
-	sprintf(buf, " [%d:%d] $end\n", msb, lsb);
+    if (bussed) {
+	sprintf(buf, " [%d:%d]", msb, lsb);
 	decl += buf;
     }
+    decl += " $end\n";
     m_namemapp->insert(make_pair(hiername,decl));
 }
 
 void SpTraceVcd::declBit (uint32_t code, const char* name, int arraynum)
-{  declare (code, name, arraynum, -1, -1); }
+{  declare (code, name, arraynum, false, 0, 0); }
 void SpTraceVcd::declBus (uint32_t code, const char* name, int arraynum, int msb, int lsb)
-{  declare (code, name, arraynum, msb, lsb); }
+{  declare (code, name, arraynum, true, msb, lsb); }
 void SpTraceVcd::declQuad  (uint32_t code, const char* name, int arraynum, int msb, int lsb)
-{  declare (code, name, arraynum, msb, lsb); }
+{  declare (code, name, arraynum, true, msb, lsb); }
 void SpTraceVcd::declArray (uint32_t code, const char* name, int arraynum, int msb, int lsb)
-{  declare (code, name, arraynum, msb, lsb); }
+{  declare (code, name, arraynum, true, msb, lsb); }
 
 //=============================================================================
 // Callbacks
