@@ -1,4 +1,4 @@
-// $Id: SpTraceVcdC.h 49154 2008-01-02 14:22:02Z wsnyder $ -*- SystemC -*-
+// $Id: SpTraceVcdC.h 61048 2008-09-18 00:45:49Z wsnyder $ -*- SystemC -*-
 //=============================================================================
 //
 // THIS MODULE IS PUBLICLY LICENSED
@@ -105,12 +105,13 @@ private:
 	}
     }
     void closePrev();
+    void closeErr();
     void openNext();
     void printIndent (int levelchange);
     void printStr (const char* str);
     void printQuad (uint64_t n);
     void printTime (uint64_t timeui);
-    void declare (uint32_t code, const char* name, int arraynum, int msb, int lsb);
+    void declare (uint32_t code, const char* name, int arraynum, bool bussed, int msb, int lsb);
 
     void dumpHeader();
     void dumpPrep (uint64_t timeui);
